@@ -1,8 +1,8 @@
-/*	$calcurse: apoint.c,v 1.18 2007/10/21 13:42:34 culot Exp $	*/
+/*	$calcurse: apoint.c,v 1.21 2008/01/20 10:45:38 culot Exp $	*/
 
 /*
  * Calcurse - text-based organizer
- * Copyright (c) 2004-2007 Frederic Culot
+ * Copyright (c) 2004-2008 Frederic Culot
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -35,7 +35,6 @@
 #include "apoint.h"
 #include "day.h"
 #include "custom.h"
-#include "utils.h"
 #include "notify.h"
 #include "recur.h"
 #include "calendar.h"
@@ -43,7 +42,8 @@
 apoint_llist_t *alist_p;
 static int	hilt = 0;
 
-int apoint_llist_init(void)
+int 
+apoint_llist_init(void)
 {
 	alist_p = (apoint_llist_t *) malloc(sizeof(apoint_llist_t));
 	alist_p->root = NULL;
@@ -79,13 +79,14 @@ apoint_hilt(void)
 }
 
 apoint_llist_node_t *
-apoint_new(char *mesg, long start, long dur, char state)
+apoint_new(char *mesg, char *note, long start, long dur, char state)
 {
 	apoint_llist_node_t *o, **i;
 
 	o = (apoint_llist_node_t *) malloc(sizeof(apoint_llist_node_t));
 	o->mesg = (char *) malloc(strlen(mesg) + 1);
 	strncpy(o->mesg, mesg, strlen(mesg) + 1);
+	o->note = (note != NULL) ? strdup(note) : NULL;
 	o->state = state;
 	o->start = start;
 	o->dur = dur;
@@ -192,12 +193,12 @@ apoint_add(void)
 			apoint_start = 
 			    date2sec(*calendar_get_slctd_day(), heures, 
 			    minutes);
-			apoint_pointeur = apoint_new(item_mesg, apoint_start,
+			apoint_pointeur = apoint_new(item_mesg, 0L, apoint_start,
 			    min2sec(apoint_duration), 0L);
 			if (notify_bar()) 
 				notify_check_added(item_mesg, apoint_start, 0L);
                 } else 
-                        event_pointeur = event_new(item_mesg, 
+                        event_pointeur = event_new(item_mesg, 0L,
 			    date2sec(*calendar_get_slctd_day(), 12, 0), Id);
 
 		if (hilt == 0) 
@@ -238,7 +239,7 @@ apoint_delete(conf_t *conf, unsigned *nb_events, unsigned *nb_apoints)
 	if (go_for_deletion) {
 		if (nb_items != 0) {
 			deleted_item_type = 
-				day_erase_item(date, hilt, 0);
+				day_erase_item(date, hilt, ERASE_DONT_FORCE);
 			if (deleted_item_type == EVNT || 
 			    deleted_item_type == RECUR_EVNT) {
 				(*nb_events)--;
@@ -250,7 +251,7 @@ apoint_delete(conf_t *conf, unsigned *nb_events, unsigned *nb_apoints)
 			} else if (deleted_item_type == 0) {
 				to_be_removed = 0;		
 			} else
-				ierror(errmsg);
+				ierror(errmsg, IERROR_FATAL);
 				/* NOTREACHED */
 
 			if (hilt > 1) 
@@ -265,8 +266,6 @@ apoint_delete(conf_t *conf, unsigned *nb_events, unsigned *nb_apoints)
 	}
 }
 
-
-
 unsigned 
 apoint_inday(apoint_llist_node_t *i, long start)
 {
@@ -319,6 +318,9 @@ apoint_write(apoint_llist_node_t *o, FILE * f)
 	    lt->tm_mon + 1, lt->tm_mday, 1900 + lt->tm_year,
 	    lt->tm_hour, lt->tm_min);
 	
+	if (o->note != NULL)
+		fprintf(f, ">%s ", o->note);
+
 	if (o->state & APOINT_NOTIFY)
 		fprintf(f, "!");
 	else
@@ -328,7 +330,7 @@ apoint_write(apoint_llist_node_t *o, FILE * f)
 }
 
 apoint_llist_node_t *
-apoint_scan(FILE * f, struct tm start, struct tm end, char state)
+apoint_scan(FILE * f, struct tm start, struct tm end, char state, char *note)
 {
 	struct tm *lt;
 	char buf[MESG_MAXSIZE], *nl;
@@ -357,29 +359,56 @@ apoint_scan(FILE * f, struct tm start, struct tm end, char state)
 		fputs(_("FATAL ERROR in apoint_scan: date error in the appointment\n"), stderr);
 		exit(EXIT_FAILURE);
 	}
-	return apoint_new(buf, tstart, tend - tstart, state);
+	return (apoint_new(buf, note, tstart, tend - tstart, state));
 }
 
-void apoint_delete_bynum(long start, unsigned num)
+/* Retrieve an appointment from the list, given the day and item position. */
+apoint_llist_node_t *
+apoint_get(long day, int pos)
+{
+	apoint_llist_node_t *o;
+	int n;
+
+	n = 0;
+	for (o = alist_p->root; o; o = o->next) {
+		if (apoint_inday(o, day)) {
+			if (n == pos)
+				return o;
+			n++;
+		}
+	}
+	/* NOTREACHED */
+	fputs(_("FATAL ERROR in apoint_get: no such item\n"), stderr);
+	exit(EXIT_FAILURE);
+}
+
+void 
+apoint_delete_bynum(long start, unsigned num, erase_flag_e flag)
 {
 	unsigned n;
 	int need_check_notify = 0;
 	apoint_llist_node_t *i, **iptr;
 
-	n = 0;
-	
+	n = 0;	
 	pthread_mutex_lock(&(alist_p->mutex));
 	iptr = &alist_p->root;
 	for (i = alist_p->root; i != 0; i = i->next) {
 		if (apoint_inday(i, start)) {
 			if (n == num) {
-				if (notify_bar()) 
-					need_check_notify = notify_same_item(i->start);	 
-				*iptr = i->next;
-				free(i->mesg);
-				free(i);
-				pthread_mutex_unlock(&(alist_p->mutex));
-				if (need_check_notify) notify_check_next_app();
+				if (flag == ERASE_FORCE_ONLY_NOTE)
+					erase_note(&i->note, flag);
+				else {
+					if (notify_bar()) 
+						need_check_notify = 
+						    notify_same_item(i->start);
+					*iptr = i->next;
+					free(i->mesg);
+					erase_note(&i->note, flag);
+					free(i);
+					pthread_mutex_unlock(&(alist_p->mutex));
+					if (need_check_notify) 
+						notify_check_next_app();
+				}
 				return;
 			}
 			n++;
@@ -389,35 +418,8 @@ void apoint_delete_bynum(long start, unsigned num)
 	pthread_mutex_unlock(&(alist_p->mutex));
 
 	/* NOTREACHED */
-	ierror(_("FATAL ERROR in apoint_delete_bynum: no such appointment"));
-}
-
-/* 
- * Print an item date in the appointment panel.
- */
-void display_item_date(WINDOW *win, int incolor, apoint_llist_node_t *i,
-			int type, long date, int y, int x)
-{
-	char a_st[100], a_end[100];
-	int recur = 0;
-
-	apoint_sec2str(i, type, date, a_st, a_end);
-	if (type == RECUR_EVNT || type == RECUR_APPT)
-		recur = 1;
-	if (incolor == 0) 
-		custom_apply_attr(win, ATTR_HIGHEST);
-	if (recur)
-		if (i->state & APOINT_NOTIFY)
-			mvwprintw(win, y, x, " *!%s -> %s", a_st, a_end);
-		else
-			mvwprintw(win, y, x, " * %s -> %s", a_st, a_end);
-	else
-		if (i->state & APOINT_NOTIFY)
-			mvwprintw(win, y, x, " -!%s -> %s", a_st, a_end);
-		else
-			mvwprintw(win, y, x, " - %s -> %s", a_st, a_end);
-	if (incolor == 0) 
-		custom_remove_attr(win, ATTR_HIGHEST);
+	ierror(_("FATAL ERROR in apoint_delete_bynum: no such appointment"),
+	    IERROR_FATAL);
 }
 
 /*
@@ -443,7 +445,8 @@ get_item_line(int item_nb, int nb_events_inday)
  * Update (if necessary) the first displayed pad line to make the
  * appointment panel scroll down next time pnoutrefresh is called. 
  */
-void apoint_scroll_pad_down(int nb_events_inday, int win_length) 
+void 
+apoint_scroll_pad_down(int nb_events_inday, int win_length) 
 {
 	int pad_last_line = 0;
 	int item_first_line = 0, item_last_line = 0;
@@ -464,7 +467,8 @@ void apoint_scroll_pad_down(int nb_events_inday, int win_length)
  * Update (if necessary) the first displayed pad line to make the
  * appointment panel scroll up next time pnoutrefresh is called. 
  */
-void apoint_scroll_pad_up(int nb_events_inday)
+void 
+apoint_scroll_pad_up(int nb_events_inday)
 {
 	int item_first_line = 0;
 
@@ -505,8 +509,8 @@ apoint_check_next(struct notify_app_s *app, long start)
  * Returns a structure of type apoint_llist_t given a structure of type 
  * recur_apoint_s 
  */
-apoint_llist_node_t *apoint_recur_s2apoint_s(
-	recur_apoint_llist_node_t *p)
+apoint_llist_node_t *
+apoint_recur_s2apoint_s(recur_apoint_llist_node_t *p)
 {
 	apoint_llist_node_t *a;
 
@@ -566,7 +570,8 @@ apoint_switch_notify(void)
 
 	/* NOTREACHED */
 	ierror(
-	    _("FATAL ERROR in apoint_switch_notify: no such appointment"));
+	    _("FATAL ERROR in apoint_switch_notify: no such appointment"),
+	    IERROR_FATAL);
 }
 
 /* Updates the Appointment panel */
