@@ -1,8 +1,8 @@
-/*	$calcurse: calcurse.c,v 1.57 2007/10/21 13:42:34 culot Exp $	*/
+/*	$calcurse: calcurse.c,v 1.60 2008/01/13 12:40:45 culot Exp $	*/
 
 /*
  * Calcurse - text-based organizer
- * Copyright (c) 2004-2007 Frederic Culot
+ * Copyright (c) 2004-2008 Frederic Culot
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -346,6 +346,24 @@ main(int argc, char **argv)
 			}
 			break;
 
+		case 'N':
+		case 'n':	
+			/* Attach a note to an item, create it if necessary */
+			if (wins_slctd() == APP && apoint_hilt() != 0)
+				day_edit_note(conf.editor);
+			else if (wins_slctd() == TOD && todo_hilt() != 0)
+				todo_edit_note(conf.editor);
+			do_storage = true;
+			break;
+
+		case '>':	
+			/* View a note previously attached to an item */
+			if (wins_slctd() == APP && apoint_hilt() != 0)
+				day_view_note(conf.pager);
+			else if (wins_slctd() == TOD && todo_hilt() != 0)
+				todo_view_note(conf.pager);
+			break;
+
 		case '?':	/* Online help system */
 			status_bar();
 			help_screen();
@@ -368,7 +386,7 @@ main(int argc, char **argv)
 			if (wins_slctd() == CAL || ch == CTRL('L')) {
 				do_storage = true;
 				day_changed = true;
-				calendar_move_right();
+				calendar_move(RIGHT);
 			}
 			break;
 
@@ -379,7 +397,7 @@ main(int argc, char **argv)
 			if (wins_slctd() == CAL || ch == CTRL('H')) {
 				do_storage = true;
 				day_changed = true;
-				calendar_move_left();
+				calendar_move(LEFT);
 			}
 			break;
 
@@ -390,7 +408,7 @@ main(int argc, char **argv)
 			if (wins_slctd() == CAL || ch == CTRL('K')) {
 				do_storage = true;
 				day_changed = true;
-				calendar_move_up();
+				calendar_move(UP);
 			} else {
 				if ((wins_slctd() == APP) && 
 				    (apoint_hilt() > 1)) {
@@ -412,7 +430,7 @@ main(int argc, char **argv)
 			if (wins_slctd() == CAL || ch == CTRL('J')) {
 				do_storage = true;
 				day_changed = true;
-				calendar_move_down();
+				calendar_move(DOWN);
 			} else {
 				if ((wins_slctd() == APP) && 
 				    (apoint_hilt() < inday.nb_events + 
