@@ -1,8 +1,8 @@
-/*	$calcurse: day.c,v 1.30 2007/10/21 13:42:34 culot Exp $	*/
+/*	$calcurse: day.c,v 1.34 2008/01/20 10:45:38 culot Exp $	*/
 
 /*
  * Calcurse - text-based organizer
- * Copyright (c) 2004-2007 Frederic Culot
+ * Copyright (c) 2004-2008 Frederic Culot
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,9 +31,9 @@
 #include <time.h>
 
 #include "i18n.h"
-#include "utils.h"
 #include "apoint.h"
 #include "event.h"
+#include "custom.h"
 #include "day.h"
 
 static struct day_item_s 	*day_items_ptr;
@@ -55,12 +55,13 @@ day_free_list(void)
 
 /* Add an event in the current day list */
 static struct day_item_s *
-day_add_event(int type, char *mesg, long day, int id)
+day_add_event(int type, char *mesg, char *note, long day, int id)
 {
 	struct day_item_s *o, **i;
 	o = (struct day_item_s *) malloc(sizeof(struct day_item_s));
 	o->mesg = (char *) malloc(strlen(mesg) + 1);
 	strncpy(o->mesg, mesg, strlen(mesg) + 1);
+	o->note = note;
 	o->type = type;
 	o->appt_dur = 0;
 	o->appt_pos = 0;
@@ -80,8 +81,8 @@ day_add_event(int type, char *mesg, long day, int id)
 
 /* Add an appointment in the current day list. */
 static struct day_item_s *
-day_add_apoint(int type, char *mesg, long start, long dur, char state, 
-    int real_pos)
+day_add_apoint(int type, char *mesg, char *note, long start, long dur, 
+    char state, int real_pos)
 {
 	struct day_item_s *o, **i;
 	int insert_item = 0;
@@ -89,6 +90,7 @@ day_add_apoint(int type, char *mesg, long start, long dur, char state,
 	o = (struct day_item_s *) malloc(sizeof(struct day_item_s));
 	o->mesg = (char *) malloc(strlen(mesg) + 1);
 	strncpy(o->mesg, mesg, strlen(mesg) + 1);
+	o->note = note;
 	o->start = start;
 	o->appt_dur = dur;
 	o->appt_pos = real_pos;
@@ -130,7 +132,8 @@ day_store_events(long date)
 	for (j = eventlist; j != 0; j = j->next) {
 		if (event_inday(j, date)) {
 			e_nb++;
-			ptr = day_add_event(EVNT, j->mesg, j->day, j->id);
+			ptr = day_add_event(EVNT, j->mesg, j->note, j->day, 
+			    j->id);
 		}	
 	}
 
@@ -153,9 +156,10 @@ day_store_recur_events(long date)
 
 	for (j = recur_elist; j != 0; j = j->next) {
 		if (recur_item_inday(j->day, j->exc, j->rpt->type, j->rpt->freq,
-			j->rpt->until, date)) {
+		    j->rpt->until, date)) {
 			e_nb++;
-			ptr = day_add_event(RECUR_EVNT, j->mesg, j->day, j->id);
+			ptr = day_add_event(RECUR_EVNT, j->mesg, j->note, 
+			    j->day, j->id);
 		}	
 	}
 
@@ -180,8 +184,8 @@ day_store_apoints(long date)
 	for (j = alist_p->root; j != 0; j = j->next) {
 		if (apoint_inday(j, date)) {
 			a_nb++;
-			ptr = day_add_apoint(APPT, j->mesg, j->start, j->dur, 
-			    j->state, 0);
+			ptr = day_add_apoint(APPT, j->mesg, j->note, j->start, 
+			    j->dur, j->state, 0);
 		}	
 	}
 	pthread_mutex_unlock(&(alist_p->mutex));
@@ -209,9 +213,8 @@ day_store_recur_apoints(long date)
 		if ((real_start = recur_item_inday(j->start, j->exc, 
 		    j->rpt->type, j->rpt->freq, j->rpt->until, date)) ){
 			a_nb++;
-			ptr = day_add_apoint(
-			    RECUR_APPT, j->mesg, real_start, j->dur, 
-			    j->state, n);
+			ptr = day_add_apoint(RECUR_APPT, j->mesg, j->note,
+			    real_start, j->dur, j->state, n);
 			n++;
 		}	
 	}
@@ -301,6 +304,63 @@ day_item_s2apoint_s(apoint_llist_node_t *a, struct day_item_s *p)
 	a->mesg = p->mesg;
 }
 
+/* 
+ * Print an item date in the appointment panel.
+ */
+static void 
+display_item_date(int incolor, apoint_llist_node_t *i, int type, long date, 
+    int y, int x)
+{
+	WINDOW *win;
+	char a_st[100], a_end[100];
+	int recur = 0;
+
+	win = apad->ptrwin;
+	apoint_sec2str(i, type, date, a_st, a_end);
+	if (type == RECUR_EVNT || type == RECUR_APPT)
+		recur = 1;
+	if (incolor == 0) 
+		custom_apply_attr(win, ATTR_HIGHEST);
+	if (recur)
+		if (i->state & APOINT_NOTIFY)
+			mvwprintw(win, y, x, " *!%s -> %s", a_st, a_end);
+		else
+			mvwprintw(win, y, x, " * %s -> %s", a_st, a_end);
+	else
+		if (i->state & APOINT_NOTIFY)
+			mvwprintw(win, y, x, " -!%s -> %s", a_st, a_end);
+		else
+			mvwprintw(win, y, x, " - %s -> %s", a_st, a_end);
+	if (incolor == 0) 
+		custom_remove_attr(win, ATTR_HIGHEST);
+}
+
+/* 
+ * Print an item description in the corresponding panel window.
+ */
+static void 
+display_item(int incolor, char *msg, int recur, int note, int len, int y, int x)
+{
+	WINDOW *win;
+	int ch_recur, ch_note;
+	char buf[len];
+
+	win = apad->ptrwin;
+	ch_recur = (recur) ? '*' : ' ';
+	ch_note = (note) ? '>' : ' ';
+	if (incolor == 0) 
+		custom_apply_attr(win, ATTR_HIGHEST);
+	if (strlen(msg) < len)
+		mvwprintw(win, y, x, " %c%c%s", ch_recur, ch_note, msg);
+	else {
+		strncpy(buf, msg, len - 1);
+		buf[len - 1] = '\0';
+		mvwprintw(win, y, x, " %c%c%s...", ch_recur, ch_note, buf);
+	}
+	if (incolor == 0) 
+		custom_remove_attr(win, ATTR_HIGHEST);
+}
+
 /* 
  * Write the appointments and events for the selected day in a pad.
  * An horizontal line is drawn between events and appointments, and the
@@ -339,8 +399,8 @@ day_write_pad(long date, int width, int length, int incolor)
 				day_saved_item->type = p->type;
 				day_saved_item->mesg = p->mesg;
 			}
-			display_item(apad->ptrwin, item_number - incolor, p->mesg, 
-				recur, width - 5, line, x_pos);
+			display_item(item_number - incolor, p->mesg, recur,
+			    (p->note != NULL) ? 1 : 0, width - 7, line, x_pos);
 			line++;
 			draw_line = true;
 		} else {
@@ -360,17 +420,19 @@ day_write_pad(long date, int width, int length, int incolor)
 				apoint_sec2str(&a, p->type, date,
 				    day_saved_item->start, day_saved_item->end);
 			}
-			display_item_date(apad->ptrwin, item_number - incolor, 
-				&a, p->type, date, line + 1, x_pos);	
-			display_item(apad->ptrwin, item_number - incolor, p->mesg,
-				0, width - 7, line + 2, x_pos + 2);
-			line = line + 3;
+			display_item_date(item_number - incolor, &a, p->type, 
+			    date, line + 1, x_pos);	
+			display_item(item_number - incolor, p->mesg, 0, 
+			    (p->note != NULL) ? 1 : 0, width - 7, line + 2, 
+			    x_pos);
+			line += 3;
 		}
 	}
 }
 
 /* Display an item inside a popup window. */
-void day_popup_item(void)
+void 
+day_popup_item(void)
 {
 	char *error = 
 		_("FATAL ERROR in day_popup_item: unknown item type\n");
@@ -382,7 +444,7 @@ void day_popup_item(void)
 		item_in_popup(day_saved_item->start, day_saved_item->end,
 			day_saved_item->mesg, _("Appointment :"));
 	else
-		ierror(error);
+		ierror(error, IERROR_FATAL);
 		/* NOTREACHED */
 }
 
@@ -390,7 +452,9 @@ void day_popup_item(void)
  * Need to know if there is an item for the current selected day inside
  * calendar. This is used to put the correct colors inside calendar panel.
  */
- int day_check_if_item(date_t day) {
+int 
+day_check_if_item(date_t day) 
+{
 	struct recur_event_s  *re;
 	recur_apoint_llist_node_t *ra;
 	struct event_s *e;
@@ -429,7 +493,8 @@ void day_popup_item(void)
 
 /* Request the user to enter a new time. */
 static char *
-day_edit_time(long time) {
+day_edit_time(long time)
+{
 	char *timestr;
 	char *msg_time = _("Enter the new time ([hh:mm] or [h:mm]) : ");
         char *enter_str = _("Press [Enter] to continue");
@@ -444,222 +509,250 @@ day_edit_time(long time) {
 			status_mesg(fmt_msg, enter_str);
 			wgetch(win[STA].p);
 		} else
-			return timestr;
+			return (timestr);
 	}
 }
 
+static void
+update_start_time(long *start, long *dur)
+{
+	long newtime;
+	unsigned hr, mn;
+	int valid_date;
+	char *timestr;
+	char *msg_wrong_time =
+	    _("Invalid time: start time must be before end time!");
+        char *msg_enter = _("Press [Enter] to continue");
+
+	do {
+		timestr = day_edit_time(*start);
+		sscanf(timestr, "%u:%u", &hr, &mn);
+		free(timestr);
+		newtime = update_time_in_date(*start, hr, mn);
+		if (newtime < *start + *dur) {
+			*dur -= (newtime - *start);
+			*start = newtime;
+			valid_date = 1;
+		} else {
+			status_mesg(msg_wrong_time, msg_enter);
+			wgetch(win[STA].p);
+			valid_date = 0;
+		}
+	} while (valid_date == 0);
+}
+
+static void
+update_duration(long *start, long *dur)
+{
+	long newtime;
+	unsigned hr, mn;
+	char *timestr;
+
+	timestr = day_edit_time(*start + *dur);
+	sscanf(timestr, "%u:%u", &hr, &mn);
+	free(timestr);
+	newtime = update_time_in_date(*start, hr, mn);
+	*dur = (newtime > *start) ? newtime - *start : 
+	    DAYINSEC + newtime - *start;
+}
+
+static void
+update_desc(char **desc)
+{
+	status_mesg(_("Enter the new item description:"), "");
+	updatestring(win[STA].p, desc, 0, 1);
+}
+
+static void
+update_rept(struct rpt_s **rpt, const long start)
+{
+	const int SINGLECHAR = 2;
+	int ch, cancel, newfreq, date_entered, valid_date;
+	long newuntil;
+	char *typstr, *freqstr, *timstr;
+	char *msg_rpt_type = 
+	    _("Enter the new repetition type: (D)aily, (W)eekly, "
+	    "(M)onthly, (Y)early");
+	char *msg_rpt_ans = _("[D/W/M/Y] ");
+	char *msg_wrong_freq = _("The frequence you entered is not valid.");
+	char *msg_wrong_time =
+	    _("Invalid time: start time must be before end time!");
+	char *msg_wrong_date = _("The entered date is not valid.");
+	char *msg_fmts = _("Possible formats are [mm/dd/yyyy] or '0' "
+	    "for an endless repetetition");
+        char *msg_enter = _("Press [Enter] to continue");
+
+	do {
+		status_mesg(msg_rpt_type, msg_rpt_ans);
+		typstr = (char *)malloc(sizeof(char) * SINGLECHAR);
+		snprintf(typstr, SINGLECHAR, "%c", recur_def2char((*rpt)->type));
+		cancel = updatestring(win[STA].p, &typstr, 0, 1);
+		if (cancel) {
+			free(typstr);
+			return;
+		} else {
+			ch = toupper(*typstr);
+			free(typstr);
+		}
+	} while ((ch != 'D') && (ch != 'W') && (ch != 'M') && (ch != 'Y'));
+
+	do {
+		status_mesg(_("Enter the new repetition frequence:"), "");
+		freqstr = (char *)malloc(BUFSIZ); 
+		snprintf(freqstr, BUFSIZ, "%d", (*rpt)->freq);
+		cancel = updatestring(win[STA].p, &freqstr, 0, 1);
+		if (cancel) {
+			free(freqstr);
+			return;
+		} else {
+			newfreq = atoi(freqstr);
+			free(freqstr);
+			if (newfreq == 0) {
+				status_mesg(msg_wrong_freq, msg_enter);
+				wgetch(win[STA].p);
+			}
+		}
+	} while (newfreq == 0);
+
+	do {
+		status_mesg(_("Enter the new ending date: [mm/dd/yyyy] or '0'"),
+		    "");
+		timstr = date_sec2date_str((*rpt)->until);
+		cancel = updatestring(win[STA].p, &timstr, 0, 1);
+		if (cancel) {
+			free(timstr);
+			return;
+		}
+		if (strcmp(timstr, "0") == 0) {
+			newuntil = 0;
+			date_entered = 1;
+		} else {
+ 			struct tm *lt;
+			time_t t;
+			date_t new_date;
+			int newmonth, newday, newyear;
+
+			valid_date = check_date(timstr);
+			if (valid_date) {
+				sscanf(timstr, "%d / %d / %d", 
+				    &newmonth, &newday, &newyear);	
+				t = start; 
+				lt = localtime(&t);
+				new_date.dd = newday;
+				new_date.mm = newmonth;
+				new_date.yyyy = newyear;
+				newuntil = date2sec(new_date, lt->tm_hour, 
+				    lt->tm_min);
+				if (newuntil < start) {
+					status_mesg(msg_wrong_time, msg_enter);
+					wgetch(win[STA].p);
+					date_entered = 0;
+				} else
+					date_entered = 1;
+			} else {
+				status_mesg(msg_wrong_date, msg_fmts);
+				wgetch(win[STA].p);
+				date_entered = 0;
+			}
+		}
+	} while (date_entered == 0);
+
+	free(timstr);
+	(*rpt)->type = recur_char2def(ch);
+	(*rpt)->freq = newfreq;
+	(*rpt)->until = newuntil;
+}
+
 /* Edit an already existing item. */
 void 
 day_edit_item(void)
 {
-#define STRT	'1'
-#define END	'2'
-#define DESC	'3'
-#define REPT	'4'
+#define STRT		'1'
+#define END		'2'
+#define DESC		'3'
+#define REPT		'4'
 
 	struct day_item_s *p;
 	struct recur_event_s *re;
-	struct rpt_s *rpt = 0L;
-	struct tm *lt;
-	time_t t;
-	date_t new_date;
-	recur_apoint_llist_node_t *ra, *ra_new;
-	long date, newtime = 0;
-	int cancel, ch = 0, valid_date = 0, newfreq = 0, date_entered = 0;
-	int newmonth, newday, newyear;
-	int item_num;
-	unsigned hr, mn;
-	char *timestr, *typestr, *freqstr;
-	char *msg_norecur =
-	_("Edit: (1)Start time, (2)End time or (3)Description?");
-	char *choice_norecur = "[1/2/3] ";
-	char *msg_recur =
-	_("Edit: (1)Start time, (2)End time, (3)Description or (4)Repetition?");
-	char *msg_event_recur =
-	_("Edit: (1)Description or (2)Repetition?");
-	char *choice_recur = "[1/2/3/4] ";
-	char *choice_event_recur = "[1/2] ";
-	char *mesg_wrong_date = _("The entered date is not valid.");
-	char *mesg_possible_fmts = 
-	_("Possible formats are [mm/dd/yyyy] or '0' for an endless repetetition");
-	char *error_msg =
-	_("Invalid time: start time must be before end time!");
-        char *enter_str = _("Press [Enter] to continue");
-	char *mesg_desc = _("Enter the new item description:");
-	char *mesg_type_1 = 
-	_("Enter the new repetition type: (D)aily, (W)eekly, (M)onthly, (Y)early");
-	char *mesg_type_2 = _("[D/W/M/Y] ");
-	char *mesg_freq_1 = _("Enter the new repetition frequence:");
-	char *mesg_wrong_freq = _("The frequence you entered is not valid.");
-	char *mesg_until_1 = 
-	_("Enter the new ending date: [mm/dd/yyyy] or '0'");
+	struct event_s *e;
+	recur_apoint_llist_node_t *ra;
+	apoint_llist_node_t *a;
+	long date;
+	int item_num, ch;
 
 	item_num = apoint_hilt();
 	p = day_get_item(item_num);
 	date = calendar_get_slctd_day_sec();
 
+	ch = 0;
 	switch (p->type) {
 	case RECUR_EVNT:
 		re = recur_get_event(date, 
 		    day_item_nb(date, item_num, RECUR_EVNT));
-		rpt = re->rpt;
-		status_mesg(msg_event_recur, choice_event_recur);
-		while (ch != STRT && ch != END && ch != ESCAPE)
+		status_mesg(_("Edit: (1)Description or (2)Repetition?"),
+		    "[1/2] ");
+		while (ch != '1' && ch != '2' && ch != ESCAPE)
 			ch = wgetch(win[STA].p);
-		if (ch == ESCAPE)
+		switch (ch) {
+		case '1':
+			update_desc(&re->mesg);
+			break;
+		case '2':
+			update_rept(&re->rpt, re->day);
+			break;
+		default:
 			return;
-		else
-			ch += 2;
+		}
 		break;
 	case EVNT:
-		ch = DESC;
+		e = event_get(date, day_item_nb(date, item_num, EVNT));
+		update_desc(&e->mesg);
 		break;
 	case RECUR_APPT:
 		ra = recur_get_apoint(date, 
 		    day_item_nb(date, item_num, RECUR_APPT));
-		rpt = ra->rpt;
-		status_mesg(msg_recur, choice_recur);
+		status_mesg(_("Edit: (1)Start time, (2)End time, "
+		    "(3)Description or (4)Repetition?"), "[1/2/3/4] ");
 		while (ch != STRT && ch != END && ch != DESC && 
-			ch != REPT && ch != ESCAPE)
+		    ch != REPT && ch != ESCAPE)
 			ch = wgetch(win[STA].p);
-		if (ch == ESCAPE)
+		switch (ch) {
+		case STRT:
+			update_start_time(&ra->start, &ra->dur);
+			break;
+		case END:
+			update_duration(&ra->start, &ra->dur);
+			break;
+		case DESC:
+			update_desc(&ra->mesg);
+			break;
+		case REPT:
+			update_rept(&ra->rpt, ra->start);
+			break;
+		case ESCAPE:
 			return;
+		}
 		break;
 	case APPT:
-		status_mesg(msg_norecur, choice_norecur);
-		while (ch != STRT && ch != END && ch != DESC && ch != ESCAPE) 
+		a = apoint_get(date, day_item_nb(date, item_num, APPT));
+		status_mesg(_("Edit: (1)Start time, (2)End time "
+		    "or (3)Description?"), "[1/2/3] ");
+		while (ch != STRT && ch != END && ch != DESC && ch != ESCAPE)
 			ch = wgetch(win[STA].p);
-		if (ch == ESCAPE)
+		switch (ch) {
+		case STRT:
+			update_start_time(&a->start, &a->dur);
+			break;
+		case END:
+			update_duration(&a->start, &a->dur);
+			break;
+		case DESC:
+			update_desc(&a->mesg);
+			break;
+		case ESCAPE:
 			return;
-		break;
-	}
-
-	switch (ch) {
-	case STRT:
-		while (!valid_date) {
-			timestr = day_edit_time(p->start);
-			sscanf(timestr, "%u:%u", &hr, &mn);
-			free(timestr);
-			newtime = update_time_in_date(p->start, hr, mn);
-			if (newtime < p->start + p->appt_dur) {
-				p->appt_dur -= (newtime - p->start);
-				p->start = newtime;
-				valid_date = 1;
-			} else {
-				status_mesg(error_msg, enter_str);
-				wgetch(win[STA].p);	
-			}
 		}
 		break;
-	case END:
-		while (!valid_date) {
-			timestr = day_edit_time(
-				p->start + p->appt_dur);
-			sscanf(timestr, "%u:%u", &hr, &mn);
-			free(timestr);
-			newtime = update_time_in_date(p->start, hr, mn);
-			p->appt_dur = (newtime > p->start) ? 
-			    newtime - p->start : 
-			    DAYINSEC + newtime - p->start;
-			valid_date = 1;
-		}
-		break;
-	case DESC:
-		status_mesg(mesg_desc, "");
-		updatestring(win[STA].p, &p->mesg, 0, 1);
-		break;
-	case REPT:
-		while ( (ch != 'D') && (ch != 'W') && (ch != 'M') 
-		    && (ch != 'Y') ) {
-			status_mesg(mesg_type_1, mesg_type_2);
-			typestr = (char *)malloc(sizeof(char) * 2);
-			*typestr = recur_def2char(rpt->type);
-			cancel = updatestring(win[STA].p, &typestr, 0, 1);
-			ch = toupper(*typestr);
-			free(typestr);
-			if (cancel)
-				return;
-		}
-		while (newfreq == 0) {
-			status_mesg(mesg_freq_1, "");
-			freqstr = (char *) malloc(BUFSIZ); 
-			snprintf(freqstr, BUFSIZ, "%d", rpt->freq);
-			cancel = updatestring(win[STA].p, &freqstr, 0, 1);
-			newfreq = atoi(freqstr);
-			free(freqstr);
-			if (cancel)	
-				return;
-			else {
-				if (newfreq == 0) {
-					status_mesg(mesg_wrong_freq, enter_str);
-					wgetch(win[STA].p);
-				}
-			}
-		}
-		while (!date_entered) {
-			status_mesg(mesg_until_1, "");
-			timestr = date_sec2date_str(rpt->until);
-			cancel = updatestring(win[STA].p, &timestr, 0, 1);
-			if (cancel) {
-				free(timestr);
-				return;
-			}
-			if (strlen(timestr) == 1 && 
-			    strncmp(timestr, "0", 1) == 0 ) {
-				rpt->until = 0;
-				date_entered = 1;
-			} else { 
-				valid_date = check_date(timestr);
-				if (valid_date) {
-					sscanf(timestr, "%d / %d / %d", 
-						&newmonth, &newday, &newyear);	
-					t = p->start; lt = localtime(&t);
-					new_date.dd = newday;
-					new_date.mm = newmonth;
-					new_date.yyyy = newyear;
-					rpt->until = date2sec(new_date,
-						lt->tm_hour, lt->tm_min);
-					if (rpt->until < p->start) {
-						status_mesg(error_msg,
-							enter_str);
-						wgetch(win[STA].p);
-						date_entered = 0;
-					} else
-						date_entered = 1;
-				} else {
-					status_mesg(mesg_wrong_date, 
-						mesg_possible_fmts);
-					wgetch(win[STA].p);
-					date_entered = 0;
-				}
-			}
-		}
-		free(timestr);
-		rpt->freq = newfreq;
-		rpt->type = recur_char2def(ch);
-		break;
-	}
-	day_erase_item(date, item_num, 1);
-
-	switch (p->type) {
-	case RECUR_EVNT:
-		recur_event_new(p->mesg, p->start, p->evnt_id, 
-			rpt->type, rpt->freq, rpt->until, NULL);
-		break;
-	case EVNT:
-		event_new(p->mesg, p->start, p->evnt_id);
-		break;
-	case RECUR_APPT:
-		ra_new = recur_apoint_new(p->mesg, p->start, p->appt_dur, 
-			p->state, rpt->type, rpt->freq, rpt->until, NULL);
-		if (notify_bar()) 
-			notify_check_repeated(ra_new);
-		break;
-	case APPT:
-		apoint_new(p->mesg, p->start, p->appt_dur, p->state);
-		if (notify_bar()) 
-			notify_check_added(p->mesg, p->start, p->state);
-		break;
 	}
 }
 
@@ -669,26 +762,44 @@ day_edit_item(void)
  * recurrent appointments and appointments) and then to test the
  * type of the item to be deleted.
  */
-int day_erase_item(long date, int item_number, int force_erase) {
-	int ch = 0;
-	unsigned delete_whole;
+int 
+day_erase_item(long date, int item_number, erase_flag_e flag)
+{
 	struct day_item_s *p;
 	char *erase_warning =
 		_("This item is recurrent. "
 		  "Delete (a)ll occurences or just this (o)ne ?");
-	char *erase_choice =
-		_("[a/o] ");
+	char *note_warning =
+		_("This item has a note attached to it. "
+		  "Delete (i)tem or just its (n)ote ?");
+	char *note_choice = _("[i/n] ");
+	char *erase_choice = _("[a/o] ");
+	int ch = 0, ans;
+	unsigned delete_whole;
 
-	if (force_erase) 
-		ch = 'a';
-
-	p = day_get_item(item_number);
-	
+	p = day_get_item(item_number);	
+	if (flag == ERASE_DONT_FORCE) {
+		ans = 0;
+		if (p->note == NULL)
+			ans = 'i';
+		while (ans != 'i' && ans != 'n') {
+			status_mesg(note_warning, note_choice);
+			ans = wgetch(win[STA].p);
+		}
+		if (ans == 'i')
+			flag = ERASE_FORCE;
+		else
+			flag = ERASE_FORCE_ONLY_NOTE;
+	}
 	if (p->type == EVNT) {
-		event_delete_bynum(date, day_item_nb(date, item_number, EVNT));
+		event_delete_bynum(date, day_item_nb(date, item_number, EVNT),
+		    flag);
 	} else if (p->type == APPT) {
-		apoint_delete_bynum(date, day_item_nb(date, item_number, APPT));
+		apoint_delete_bynum(date, day_item_nb(date, item_number, APPT),
+		    flag);
 	} else {
+		if (flag == ERASE_FORCE_ONLY_NOTE)
+			ch = 'a';
 		while ( (ch != 'a') && (ch != 'o') && (ch != ESCAPE)) {
 			status_mesg(erase_warning, erase_choice);
 			ch = wgetch(win[STA].p);
@@ -703,12 +814,13 @@ int day_erase_item(long date, int item_number, int force_erase) {
 		if (p->type == RECUR_EVNT) {
 			recur_event_erase(date, 
 			    day_item_nb(date, item_number, RECUR_EVNT), 
-			    delete_whole);
+			    delete_whole, flag);
 		} else {
-			recur_apoint_erase(date, p->appt_pos, delete_whole);
+			recur_apoint_erase(date, p->appt_pos, delete_whole, 
+			    flag);
 		}
 	}
-	return p->type;
+	return (p->type);
 }
 
 /* Returns a structure containing the selected item. */
@@ -743,3 +855,65 @@ day_item_nb(long date, int day_num, int type)
 
 	return (nb_item[type - 1]);
 }
+
+/* Attach a note to an appointment or event. */
+void
+day_edit_note(char *editor)
+{
+	struct day_item_s *p;
+	recur_apoint_llist_node_t *ra;
+	apoint_llist_node_t *a;
+	struct recur_event_s *re;
+	struct event_s *e;
+	char fullname[BUFSIZ];
+	char *filename;
+	long date;
+	int item_num;
+
+	item_num = apoint_hilt();
+	p = day_get_item(item_num);
+	if (p->note == NULL) {
+		if ((filename = new_tempfile(path_notes, NOTESIZ)) == NULL)
+			return;
+		else
+			p->note = filename;
+	}
+	snprintf(fullname, BUFSIZ, "%s%s", path_notes, p->note);
+	wins_launch_external(fullname, editor);
+
+	date = calendar_get_slctd_day_sec();
+	switch (p->type) {
+	case RECUR_EVNT:
+		re = recur_get_event(date, 
+		    day_item_nb(date, item_num, RECUR_EVNT));
+		re->note = p->note;
+		break;
+	case EVNT:
+		e = event_get(date, day_item_nb(date, item_num, EVNT));
+		e->note = p->note;
+		break;
+	case RECUR_APPT:
+		ra = recur_get_apoint(date, 
+		    day_item_nb(date, item_num, RECUR_APPT));
+		ra->note = p->note;
+		break;
+	case APPT:
+		a = apoint_get(date, day_item_nb(date, item_num, APPT));
+		a->note = p->note;
+		break;
+	}
+}
+
+/* View a note previously attached to an appointment or event */
+void 
+day_view_note(char *pager)
+{
+	struct day_item_s *p;
+	char fullname[BUFSIZ];
+
+	p = day_get_item(apoint_hilt());
+	if (p->note == NULL)
+		return;
+	snprintf(fullname, BUFSIZ, "%s%s", path_notes, p->note);
+	wins_launch_external(fullname, pager);
+}
