@@ -1,8 +1,8 @@
-/*	$calcurse: day.h,v 1.15 2007/08/15 15:37:10 culot Exp $	*/
+/*	$calcurse: day.h,v 1.17 2008/01/20 10:45:38 culot Exp $	*/
 
 /*
  * Calcurse - text-based organizer
- * Copyright (c) 2004-2007 Frederic Culot
+ * Copyright (c) 2004-2008 Frederic Culot
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,6 +27,7 @@
 #ifndef CALCURSE_DAY_H
 #define CALCURSE_DAY_H
 
+#include "utils.h"
 #include "calendar.h"
 
 #define MAX_TYPES	4
@@ -50,6 +51,7 @@ struct day_item_s {
 	int appt_pos;		/* real position in recurrent list */
 	char state;		/* appointment state */
 	char *mesg;		/* item description */
+	char *note;		/* note attached to item */
 };
 
 struct day_saved_item_s {
@@ -65,8 +67,10 @@ void 			day_write_pad(long, int, int, int);
 void 			day_popup_item(void);
 int	 		day_check_if_item(date_t);
 void 			day_edit_item(void);
-int 			day_erase_item(long, int, int);
+int 			day_erase_item(long, int, erase_flag_e);
 struct day_item_s      *day_get_item(int);
 int 			day_item_nb(long, int, int);
+void			day_edit_note(char *);
+void			day_view_note(char *);
 
 #endif /* CALCURSE_DAY_H */
