@@ -1,8 +1,8 @@
-/*	$calcurse: event.h,v 1.2 2007/07/28 13:11:42 culot Exp $	*/
+/*	$calcurse: event.h,v 1.5 2008/01/20 10:45:38 culot Exp $	*/
 
 /*
  * Calcurse - text-based organizer
- * Copyright (c) 2004-2006 Frederic Culot
+ * Copyright (c) 2004-2008 Frederic Culot
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,6 +27,8 @@
 #ifndef CALCURSE_EVENT_H
 #define CALCURSE_EVENT_H
 
+#include "utils.h"
+
 #define HRMIN_SIZE 6
 #define MESG_MAXSIZE 256
 
@@ -35,14 +37,16 @@ struct event_s {
 	int id;	        /* event identifier */
         long day;	/* seconds since 1 jan 1970 */
 	char *mesg;
+	char *note;
 };
 
 extern struct event_s *eventlist;
 
-struct event_s *event_new(char *, long, int);
+struct event_s *event_new(char *, char *, long, int);
 unsigned 	event_inday(struct event_s *, long);
 void 		event_write(struct event_s *, FILE *);
-struct event_s *event_scan(FILE *, struct tm, int);
-void 		event_delete_bynum(long, unsigned);
+struct event_s *event_scan(FILE *, struct tm, int, char *);
+struct event_s *event_get(long, int);
+void 		event_delete_bynum(long, unsigned, erase_flag_e);
 
 #endif /* CALCURSE_EVENT_H */
