@@ -1,8 +1,8 @@
-/*	$calcurse: io.c,v 1.23 2007/10/21 13:42:34 culot Exp $	*/
+/*	$calcurse: io.c,v 1.27 2008/01/20 10:45:38 culot Exp $	*/
 
 /*
  * Calcurse - text-based organizer
- * Copyright (c) 2004-2007 Frederic Culot
+ * Copyright (c) 2004-2008 Frederic Culot
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -30,6 +30,7 @@
 #include <time.h>
 #include <math.h>
 #include <unistd.h>
+#include <errno.h>
 
 #include "i18n.h"
 #include "utils.h"
@@ -125,6 +126,7 @@ io_get_export_stream(void)
 	    _("The file cannot be accessed, please enter another file name.");
 	char *press_enter =
 	    _("Press [ENTER] to continue.");
+	int cancel;
 
 	stream = NULL;
 	stream_name = (char *)malloc(BUFSIZ);
@@ -135,7 +137,11 @@ io_get_export_stream(void)
 	
 	while (stream == NULL) {
 		status_mesg(question, "");
-		updatestring(win[STA].p, &stream_name, 0, 1);
+		cancel = updatestring(win[STA].p, &stream_name, 0, 1);
+		if (cancel) {
+			free(stream_name);
+			return (NULL);
+		}
 		stream = fopen(stream_name, "w");
 		if (stream == NULL) {
 			status_mesg(wrong_name, press_enter);
@@ -327,6 +333,7 @@ io_init(char *cfile)
 	snprintf(path_dir, BUFSIZ, "%s/" DIR_NAME, home);
 	snprintf(path_todo, BUFSIZ, "%s/" TODO_PATH, home);
 	snprintf(path_conf, BUFSIZ, "%s/" CONF_PATH, home);
+	snprintf(path_notes, BUFSIZ, "%s/" NOTES_DIR, home);
 	if (cfile == NULL) {
 		snprintf(path_apts, BUFSIZ, "%s/" APTS_PATH, home);
 	} else {
@@ -485,13 +492,19 @@ io_save_cal(conf_t *conf)
 	}
 
 	/* Save the todo data file. */
-	if (show_bar) progress_bar(PROGRESS_BAR_SAVE, 1);
+	if (show_bar) 
+		progress_bar(PROGRESS_BAR_SAVE, 1);
 	data_file = fopen(path_todo, "w");
 	if (data_file == (FILE *) 0)
 	        status_mesg(access_pb, ""); 
 	else {
-		for (i = todolist; i != 0; i = i->next)
-			fprintf(data_file, "[%d] %s\n", i->id, i->mesg);
+		for (i = todolist; i != 0; i = i->next) {
+			if (i->note != NULL)
+				fprintf(data_file, "[%d]>%s %s\n", i->id, 
+				    i->note, i->mesg);
+			else
+				fprintf(data_file, "[%d] %s\n", i->id, i->mesg);
+		}
 		fclose(data_file);
 	}
 
@@ -500,7 +513,8 @@ io_save_cal(conf_t *conf)
          * appointments first, and then the events. 
 	 * Recursive items are written first.
          */
-	if (show_bar) progress_bar(PROGRESS_BAR_SAVE, 2);
+	if (show_bar) 
+		progress_bar(PROGRESS_BAR_SAVE, 2);
 	data_file = fopen(path_apts, "w");
 	if (data_file == (FILE *) 0)
 	        status_mesg(access_pb, "");
@@ -541,6 +555,7 @@ io_load_app(void)
         int id = 0;
 	int freq;
 	char type, state = 0L;
+	char note[NOTESIZ + 1], *notep;
 	char *error = 
 		_("FATAL ERROR in io_load_app: wrong format in the appointment or event\n");
 
@@ -634,6 +649,18 @@ io_load_app(void)
 		} else 
 			ungetc(c, data_file);
 
+		/* Check if a note is attached to the item. */
+		c = getc(data_file);
+		if (c == '>') {
+			fgets(note, NOTESIZ + 1, data_file);
+			note[NOTESIZ] = '\0';
+			notep = note;
+			getc(data_file);
+		} else {
+			notep = NULL;
+			ungetc(c, data_file);
+		}
+
 		/*
 		 * Last: read the item description and load it into its
 		 * corresponding linked list, depending on the item type.
@@ -651,16 +678,17 @@ io_load_app(void)
 			}
 			if (is_recursive) {
 				recur_apoint_scan(data_file, start, end,
-				    type, freq, until, exc, state);
+				    type, freq, until, notep, exc, state);
 			} else {
-				apoint_scan(data_file, start, end, state);
+				apoint_scan(data_file, start, end, state, 
+				    notep);
 			}
 		} else if (is_event) {
 			if (is_recursive) {
-				recur_event_scan(data_file, start, id,
-					type, freq, until, exc);
+				recur_event_scan(data_file, start, id, type, 
+				    freq, until, notep, exc);
 			} else {
-				event_scan(data_file, start, id);
+				event_scan(data_file, start, id, notep);
 			}
 		} else { /* NOT REACHED */
 			fputs(error, stderr);
@@ -680,7 +708,7 @@ io_load_todo(void)
 	char *nl;
 	int nb_tod = 0;
 	int c, id;
-	char buf[BUFSIZ], e_todo[BUFSIZ];
+	char buf[BUFSIZ], e_todo[BUFSIZ], note[NOTESIZ + 1];
 
 	data_file = fopen(path_todo, "r");
 	if (data_file == NULL) {
@@ -692,68 +720,90 @@ io_load_todo(void)
 		if (c == EOF) {
 			break;
 		} else if (c == '[') { /* new style with id */
-			fscanf(data_file, "%d] ", &id);
+			fscanf(data_file, "%d]", &id);
 		} else {
 			id = 9;
 			ungetc(c, data_file);
 		}
+		/* Now read the attached note, if any. */
+		c = getc(data_file);
+		if (c == '>') {
+			fgets(note, NOTESIZ + 1, data_file);
+			note[NOTESIZ] = '\0';
+			getc(data_file);
+		} else
+			note[0] = '\0';
+		/* Then read todo description. */
 		fgets(buf, BUFSIZ, data_file);
 		nl = strchr(buf, '\n');
 		if (nl) {
 			*nl = '\0';
 		}
 		io_extract_data(e_todo, buf, strlen(buf));
-		todo_add(e_todo, id);
+		todo_add(e_todo, id, note);
 		++nb_tod;
 	}
 	fclose(data_file);
 	todo_set_nb(nb_tod);
 }
 
-/* Checks if data files exist. If not, create them */
+static void
+check_directory(char *dir, int *missing)
+{
+	errno = 0;
+	if (mkdir(dir, 0700) != 0) {
+		if (errno != EEXIST) {
+			fprintf(stderr, 
+			    _("FATAL ERROR: could not create %s: %s\n"),
+			    dir, strerror(errno));
+			exit(EXIT_FAILURE);
+		}
+	} else
+		(*missing)++;
+}
+
+static void
+check_file(char *file, int *missing)
+{
+	FILE *fd;
+
+	errno = 0;
+	if ((fd = fopen(file, "r")) == NULL) {
+		(*missing)++;
+		if ((fd = fopen(file, "w")) == NULL) {
+			fprintf(stderr, 
+			    _("FATAL ERROR: could not create %s: %s\n"),
+			    file, strerror(errno));
+			exit(EXIT_FAILURE);
+		}
+	}
+	fclose(fd);
+}
+
+/* 
+ * Checks if data files exist. If not, create them. 
+ * The following structure has to be created:
+ *
+ *	$HOME/.calcurse/
+ *                 |
+ *                 +--- notes/
+ *                 |___ conf
+ *                 |___ apts
+ *                 |___ todo
+ */
 int 
 io_check_data_files(void)
 {
-	FILE *data_file;
-	int no_data_file;
+	int missing;
 
-	no_data_file = 0;
-	/* Create the calcurse repertory if not present. */
-	mkdir(path_dir, 0700);
-
-	data_file = fopen(path_todo, "r");
-	if (data_file == NULL) {
-		no_data_file++;
-		data_file = fopen(path_todo, "w");
-		if (data_file == NULL) {
-			perror(path_todo);
-			return no_data_file;
-		}
-	}
-	fclose(data_file);
-
-	data_file = fopen(path_apts, "r");
-	if (data_file == NULL) {
-		no_data_file++;
-		data_file = fopen(path_apts, "w");
-		if (data_file == NULL) {
-			perror(path_apts);
-			return no_data_file;
-		}
-	}
-	fclose(data_file);
-
-	data_file = fopen(path_conf, "r");
-	if (data_file == NULL) {
-		no_data_file++;
-		data_file = fopen(path_conf, "w");
-		if (data_file == NULL) {
-			perror(path_conf);
-			return no_data_file;
-		}
-	}
-	fclose(data_file);
-        return no_data_file;
+	missing = 0;
+	errno = 0;
+	check_directory(path_dir, &missing);
+	check_directory(path_notes, &missing);
+	check_file(path_todo, &missing);
+	check_file(path_apts, &missing);
+	check_file(path_conf, &missing);
+        return (missing);
 }
 
 /* Draw the startup screen */
@@ -797,6 +847,9 @@ io_export_data(export_mode_t mode, conf_t *conf)
 		/* NOTREACHED */
 	}
 
+	if (stream == NULL)
+		return;
+
 	io_export_header(stream);
 
 	if (!conf->skip_progress_bar && mode == IO_EXPORT_INTERACTIVE)
