@@ -1,8 +1,8 @@
-/*	$calcurse: io.h,v 1.1.1.1 2006/07/31 21:00:03 culot Exp $	*/
+/*	$calcurse: io.h,v 1.8 2007/08/15 15:36:27 culot Exp $	*/
 
 /*
  * Calcurse - text-based organizer
- * Copyright (c) 2004-2006 Frederic Culot
+ * Copyright (c) 2004-2007 Frederic Culot
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,16 +27,21 @@
 #ifndef CALCURSE_IO_H
 #define CALCURSE_IO_H
 
-void io_init(char *cfile);
-void extract_data(char *dst_data, const char *org, int len);
-void save_cal(bool auto_save, bool confirm_quit, 
-	 bool confirm_delete, bool skip_system_dialogs, 
-         bool skip_progress_bar, bool week_begins_on_monday, 
-	 int colr, int layout);
-void load_app();
-int load_todo(int colr);
-int check_data_files();
-void startup_screen(bool skip_dialogs, int no_data_file, int colr);
-void progress_bar(bool save, int progress);
+#include "vars.h"
+
+typedef enum {
+	IO_EXPORT_NONINTERACTIVE,
+	IO_EXPORT_INTERACTIVE,
+	IO_EXPORT_NBMODES
+} export_mode_t;
+
+void 	io_init(char *);
+void 	io_extract_data(char *, const char *, int);
+void 	io_save_cal(conf_t *); 
+void 	io_load_app(void);
+void 	io_load_todo(void);
+int 	io_check_data_files(void);
+void 	io_startup_screen(bool, int);
+void 	io_export_data(export_mode_t, conf_t *);
 
 #endif /* CALCURSE_IO_H */
