@@ -1,4 +1,4 @@
-/*	$calcurse: todo.c,v 1.15 2007/10/21 13:42:34 culot Exp $	*/
+/*	$calcurse: todo.c,v 1.19 2008/01/20 10:45:39 culot Exp $	*/
 
 /*
  * Calcurse - text-based organizer
@@ -26,8 +26,10 @@
 
 #include <stdlib.h>
 #include <string.h>
+#include <unistd.h>
 
 #include "utils.h"
+#include "custom.h"
 #include "i18n.h"
 #include "todo.h"
 
@@ -37,6 +39,20 @@ static int	todos = 0;
 static int	first = 1;
 static char    *msgsav;
 
+/* Returns a structure containing the selected item. */
+static struct todo_s *
+todo_get_item(int item_number)
+{
+	struct todo_s *o;
+	int i;
+	
+	o = todolist;
+	for (i = 1; i < item_number; i++) {
+		o = o->next;
+	}
+	return o;
+}
+
 /* Sets which todo is highlighted. */
 void
 todo_hilt_set(int highlighted)
@@ -130,20 +146,21 @@ todo_new_item(void)
 			status_mesg(mesg_id, "");
 			ch = wgetch(win[STA].p);
 		}
-		todo_add(todo_input, ch - '0');
+		todo_add(todo_input, ch - '0', NULL);
 		todos++;
 	}
 }
 
 /* Add an item in the todo linked list. */
 struct todo_s *
-todo_add(char *mesg, int id)
+todo_add(char *mesg, int id, char *note)
 {
 	struct todo_s *o, **i;
 	o = (struct todo_s *) malloc(sizeof(struct todo_s));
 	o->mesg = (char *) malloc(strlen(mesg) + 1);
 	strncpy(o->mesg, mesg, strlen(mesg) + 1);
 	o->id = id;
+	o->note = (note != NULL && note[0] != '\0') ? strdup(note) : NULL;
 	i = &todolist;
 	for (;;) {
 		if (*i == 0 || (*i)->id > id) {
@@ -156,6 +173,33 @@ todo_add(char *mesg, int id)
 	return o;
 }
 
+/* Delete a note previously attached to a todo item. */
+static void 
+todo_delete_note_bynum(unsigned num)
+{
+	unsigned n;
+	struct todo_s *i, **iptr;
+
+	n = 0;
+	iptr = &todolist;
+	for (i = todolist; i != 0; i = i->next) {
+		if (n == num) {
+			if (i->note == NULL)
+				ierror(
+				   _("FATAL ERROR in todo_delete_note_bynum: "
+				   "no note attached\n"), IERROR_FATAL);
+			erase_note(&i->note, ERASE_FORCE_ONLY_NOTE);
+			return;
+		}
+		iptr = &i->next;
+		n++;
+	}
+	/* NOTREACHED */
+	ierror(_("FATAL ERROR in todo_delete_note_bynum: no such todo\n"),
+	    IERROR_FATAL);
+	exit(EXIT_FAILURE);
+}
+
 /* Delete an item from the todo linked list. */
 static void 
 todo_delete_bynum(unsigned num)
@@ -169,6 +213,8 @@ todo_delete_bynum(unsigned num)
 		if (n == num) {
 			*iptr = i->next;
 			free(i->mesg);
+			if (i->note != NULL)
+				erase_note(&i->note, ERASE_FORCE);
 			free(i);
 			return;
 		}
@@ -176,7 +222,8 @@ todo_delete_bynum(unsigned num)
 		n++;
 	}
 	/* NOTREACHED */
-	fputs(_("FATAL ERROR in todo_delete_bynum: no such todo\n"), stderr);
+	ierror(_("FATAL ERROR in todo_delete_bynum: no such todo\n"), 
+	    IERROR_FATAL);
 	exit(EXIT_FAILURE);
 }
 
@@ -186,8 +233,13 @@ todo_delete(conf_t *conf)
 {
 	char *choices = "[y/n] ";
 	char *del_todo_str = _("Do you really want to delete this task ?");
+	char *erase_warning =
+		_("This item has a note attached to it. "
+		  "Delete (t)odo or just its (n)ote ?");
+	char *erase_choice =
+		_("[t/n] ");
 	bool go_for_todo_del = false;
-	int answer = 0;
+	int answer = 0, has_note;
 	
 	if (conf->confirm_delete) {
 		status_mesg(del_todo_str, choices);
@@ -202,7 +254,23 @@ todo_delete(conf_t *conf)
 		if (todos > 0) 
 			go_for_todo_del = true;
 
-	if (go_for_todo_del) {
+	if (go_for_todo_del == false) {
+		erase_status_bar();
+		return;
+	}
+
+	answer = 0;
+	has_note = (todo_get_item(hilt)->note != NULL) ? 1 : 0;
+	if (has_note == 0)
+		answer = 't';
+	
+	while (answer != 't' && answer != 'n' && answer != ESCAPE) {
+		status_mesg(erase_warning, erase_choice);
+		answer = wgetch(win[STA].p);
+	}
+	
+	switch (answer) {
+	case 't':
 		todo_delete_bynum(hilt - 1);
 		todos--;
 		if (hilt > 1) 
@@ -211,6 +279,13 @@ todo_delete(conf_t *conf)
 			hilt = 0;
 		if (hilt - first < 0)
 			first--;
+		break;
+	case 'n':
+		todo_delete_note_bynum(hilt - 1);
+		break;
+	default:
+		erase_status_bar();
+		return;
 	}
 }
 
@@ -240,20 +315,6 @@ todo_get_position(struct todo_s *i)
 	} 
 }
 
-/* Returns a structure containing the selected item. */
-static struct todo_s *
-todo_get_item(int item_number)
-{
-	struct todo_s *o;
-	int i;
-	
-	o = todolist;
-	for (i = 1; i < item_number; i++) {
-		o = o->next;
-	}
-	return o;
-}
-
 /* Change an item priority by pressing '+' or '-' inside TODO panel. */
 void
 todo_chg_priority(int action)
@@ -261,11 +322,13 @@ todo_chg_priority(int action)
 	struct todo_s *backup;
 	char backup_mesg[BUFSIZ];
 	int backup_id;
+	char backup_note[NOTESIZ + 1];
 	int do_chg = 1;
 
 	backup = todo_get_item(hilt);
 	strncpy(backup_mesg, backup->mesg, strlen(backup->mesg) + 1);
 	backup_id = backup->id;
+	strncpy(backup_note, backup->note, NOTESIZ + 1);
 	if (action == '+') {
 		(backup_id > 1) ? backup_id-- : do_chg--;
 	} else if (action == '-') {
@@ -276,7 +339,7 @@ todo_chg_priority(int action)
 	}	
 	if (do_chg) {
 		todo_delete_bynum(hilt - 1);
-		backup = todo_add(backup_mesg, backup_id);
+		backup = todo_add(backup_mesg, backup_id, backup_note);
 		hilt = todo_get_position(backup);	
 	} 
 }
@@ -293,12 +356,36 @@ todo_edit_item(void)
 	updatestring(win[STA].p, &i->mesg, 0, 1);
 }
 
+/* Display todo items in the corresponding panel. */
+static void
+display_todo_item(int incolor, char *msg, int prio, int note, int len, int y,
+    int x)
+{
+	WINDOW *w;
+	int ch_note;
+	char buf[len];
+
+	w = win[TOD].p;
+	ch_note = (note) ? '>' : '.';
+	if (incolor == 0) 
+		custom_apply_attr(w, ATTR_HIGHEST);
+	if (strlen(msg) < len)
+		mvwprintw(w, y, x, "%d%c %s", prio, ch_note, msg);
+	else {
+		strncpy(buf, msg, len - 1);
+		buf[len - 1] = '\0';
+		mvwprintw(w, y, x, "%d%c %s...", prio, ch_note, buf);
+	}
+	if (incolor == 0) 
+		custom_remove_attr(w, ATTR_HIGHEST);
+}
+
 /* Updates the ToDo panel. */
 void 
 todo_update_panel(window_t *wintod, int which_pan)
 {
 	struct todo_s *i;
-	int len = wintod->w - 6;
+	int len = wintod->w - 8;
 	int num_todo = 0;
 	int y_offset = 3, x_offset = 1;
 	int t_realpos = -1;
@@ -306,7 +393,6 @@ todo_update_panel(window_t *wintod, int which_pan)
 	int todo_lines = 1;
 	int max_items = wintod->h - 4;
 	int incolor = -1;
-	char mesg[BUFSIZ] = "";
 
 	/* Print todo item in the panel. */
 	erase_window_part(win[TOD].p, 1, title_lines, wintod->w - 2, 
@@ -318,10 +404,8 @@ todo_update_panel(window_t *wintod, int which_pan)
 		if (incolor == 0) 
 			msgsav = i->mesg; 
 		if (t_realpos >= 0 && t_realpos < max_items) {
-			snprintf(mesg, BUFSIZ, "%d. ", i->id);	
-			strncat(mesg, i->mesg, strlen(i->mesg));
-			display_item(win[TOD].p, incolor, mesg, 0, 
-					len, y_offset, x_offset);
+			display_todo_item(incolor, i->mesg, i->id,
+			    (i->note != NULL) ? 1 : 0, len, y_offset, x_offset);
 			y_offset = y_offset + todo_lines;	
 		}
 	}
@@ -342,3 +426,37 @@ todo_update_panel(window_t *wintod, int which_pan)
 	
 	wnoutrefresh(win[TOD].p);
 }
+
+/* Attach a note to a todo */
+void 
+todo_edit_note(char *editor)
+{
+	struct todo_s *i;
+	char fullname[BUFSIZ];
+	char *filename;
+
+	i = todo_get_item(hilt);
+	if (i->note == NULL) {
+		if ((filename = new_tempfile(path_notes, NOTESIZ))
+		    != NULL)
+			i->note = filename;
+		else
+			return;
+	}
+	snprintf(fullname, BUFSIZ, "%s%s", path_notes, i->note);
+	wins_launch_external(fullname, editor);
+}
+
+/* View a note previously attached to a todo */
+void 
+todo_view_note(char *pager)
+{
+	struct todo_s *i;
+	char fullname[BUFSIZ];
+
+	i = todo_get_item(hilt);
+	if (i->note == NULL)
+		return;
+	snprintf(fullname, BUFSIZ, "%s%s", path_notes, i->note);
+	wins_launch_external(fullname, pager);
+}
