@@ -1,4 +1,4 @@
-/*	$Id$	*/
+/*	$calcurse: wins.c,v 1.12 2008/02/14 20:20:23 culot Exp $	*/
 
 /*
  * Calcurse - text-based organizer
@@ -25,6 +25,7 @@
  */
 
 #include <stdlib.h>
+#include <string.h>
 
 #include "i18n.h"
 #include "notify.h"
@@ -337,7 +338,8 @@ wins_update(void)
 		break;
 
 	default:
-		ierror(_("FATAL ERROR in wins_update: no window selected\n"));
+		ierror(_("FATAL ERROR in wins_update: no window selected\n"),
+		    IERROR_FATAL);
 		/* NOTREACHED */
 	}
 
@@ -361,3 +363,36 @@ wins_reset(void)
 	wins_reinit();
 	wins_update();
 }
+
+/*
+ * While inside interactive mode, launch the external command cmd on the given
+ * file.
+ */
+void
+wins_launch_external(const char *file, const char *cmd)
+{
+	char *p;
+	int len;
+
+	len = strlen(file) + strlen(cmd) + 2; /* Beware of space between cmd and
+						 file. */
+	p = (char *)malloc(sizeof(char) * len);
+	if (snprintf(p, len, "%s %s", cmd, file) == -1) {
+		free(p);
+		return;
+	}
+	if (notify_bar())
+		notify_stop_main_thread();
+	def_prog_mode();
+	endwin();
+	clear();
+	refresh();
+	system(p);
+	reset_prog_mode();
+	clearok(curscr, TRUE);
+	curs_set(0);
+	refresh();
+	if (notify_bar())
+		notify_start_main_thread();
+	free(p);
+}
